# This module contains routines that parse the XML
# output string from the client program
import sys
from xml.dom.minidom import *

# Create a dom tree and startNode defined inside the
# from the input XML string
def readXMLResultFile(file):

	docObject = parse(file)
	startNode = docObject.firstChild
	
	return (docObject, startNode)

def readXMLString(xmlData):

	docObject = parseString(xmlData)
	startNode = docObject.firstChild
	
	return (docObject, startNode)

def readXMLStringLines(xmlLines):

	buffer = ''
	for line in xmlLines:
		buffer = buffer + line
	return readXMLString(buffer)

def parseXML(filename, xmlString, xmlLines):

	if filename != None:
		(doc, start) = readXMLResultFile(filename)
	elif xmlString != None:
		(doc, start) = readXMLString(xmlString)
	elif xmlLines != None:
		(doc, start) = readXMLStringLines(xmlLines)
	else:
		(doc, start) = (None, None)

	return (doc, start)
	
# If you don't know where to start parsing, then
# input docObject and will search for the whole
# top level dom tree. Otherwise gives the startNode
# and it will speed up the process
# Returns elements contain ClientResult tag
def getResultElements(docObject, startNode, cmdName):

	if docObject != None:
		start = docObject
	elif startNode != Node:
		start = startNode

	nodeList = start.getElementsByTagName('ClientResult')

	if nodeList and len(nodeList):
		for node in nodeList:
			# Should be only one command element inside the
			# ClientResult node
			commandElement = node.getElementsByTagName('Command')
			if len(commandElement) != 1:
				sys.stderr.write('Error: expect only one command element\n')
				return None

			# Matching the name attribute inside the Command element
			result = commandElement[0].getAttribute('name')
			if (result != None and len(result) and
				result == cmdName):

				# Get the Result node inside the same ClientResult
				# node
				resultElement = node.getElementsByTagName('Result')
				if len(resultElement) != 1:
					sys.stderr.write('Error: expect only one result element\n')
					return None
				else:
					return resultElement[0]

	return None

def getNodeText(node, tagName):
   	rtnCodeElem = node.getElementsByTagName(tagName)

	returnCode = None
   	if rtnCodeElem != None and len(rtnCodeElem):
		returnCode = getText(rtnCodeElem[0])

	return returnCode

def getErrorMsg(node):

	return getNodeText(node, 'Error')

def getText(node):

	textData = ""
	if node.childNodes != None and len(node.childNodes):
		for child in node.childNodes:
			if child.nodeType == node.TEXT_NODE:
				textData = textData + child.data

	return textData

def getReturnCode(node):
	return int(getNodeText(node, 'ReturnCode'))

def getOutput(node):
	return getNodeText(node, 'Output')
	
# Returns (returnCode, { queueName:len}, errorMsg)
def formatQueueInfoResult(node):

	queueResult = {}
	returnCode = -1
	msg = None
	
	if node != None:
		outputNode = node.getElementsByTagName('Output')
		
		returnCode = getReturnCode(node)
		queueNodes = outputNode[0].getElementsByTagName('Queue')
		for qNode in queueNodes:
			length = qNode.getAttribute('length')
			queueResult[getText(qNode)] = length

		msg = getErrorMsg(node)

	return (returnCode, queueResult, msg)

# Returns (returnCode, { name:value, }, errorMsg)
def formatShowConfigResult(node):

	queueResult = {}
	returnCode = -1
	msg = None
	
	if node != None:
		outputNode = node.getElementsByTagName('Output')
		
		returnCode = getReturnCode(node)
		optNodes = outputNode[0].getElementsByTagName('Option')
		for oNode in optNodes:
			nameElems = oNode.getElementsByTagName('Name')
			valueElems = oNode.getElementsByTagName('Value')
			if len(nameElems) and len(valueElems):
				name = getText(nameElems[0])
				value = getText(valueElems[0])
				queueResult[name] = value
			
		msg = getErrorMsg(node)

	return (returnCode, queueResult, msg)

def formatPollCommandResult(node):

	if node != None:
		outputNode = node.getElementsByTagName('Output')
		
		returnCode = getReturnCode(node)
		outputText = getText(outputNode[0])
		msg = getErrorMsg(node)

	return (returnCode, outputText, msg)

def parsePollCommandXMLResult(filename, xmlString, xmlLines):

	(doc, start) = parseXML(filename, xmlString, xmlLines)
	if doc != None:
		resultElement = getResultElements(None, start, 'POLL')
		return formatPollCommandResult(resultElement)

	return None
	
def parseShowConfigXMLResult(filename, xmlString, xmlLines):
	
	(doc, start) = parseXML(filename, xmlString, xmlLines)
	if doc != None:
		resultElement = getResultElements(None, start, 'SHOW_CONFIG')
		return formatShowConfigResult(resultElement)

	return None

# Highest level procedure call to parse the
# XML output for QUEUE_INFO result
# This function accepts either filename or XML string
def parseQueueInfoXMLResult(filename, xmlString, xmlLines):

	(doc, start) = parseXML(filename, xmlString, xmlLines)
	if doc != None:
		resultElement = getResultElements(None, start, 'QUEUE_INFO')
		return formatQueueInfoResult(resultElement)

	return None

# Given a document object, this procedure parse all the
# XML log elements and return a list of tuples in a
# sequence of (time, level, levelName, message(s))
def getLogMessages(docObject):

	logMessages = []

	nodeList = docObject.getElementsByTagName('Log')
	for node in nodeList:
		time = node.getAttribute('time')
		level = node.getAttribute('level')
		levelName = node.getAttribute('levelName')
		messages = getText(node)

		logMessages.append((time, level, levelName, messages))

	return logMessages

# Returns the CDATA inside the Output element of 
def getEventExecuteResultOutput(filename, xmlString, xmlLines):
	
	(doc, start) = parseXML(filename, xmlString, xmlLines)
	if doc != None:
		resultElement = getResultElements(None, start, 'EXECUTE_EVENT')
		if resultElement != None:
			return getOutput(resultElement)
			
	return None

def getEventExecuteReturnCode(filename, xmlString, xmlLines):

	return getEventReturnCode(filename, xmlString,
							  xmlLines, 'EXECUTE_EVENT')

def getEventFileSendReturnCode(filename, xmlString, xmlLines):

	return getEventReturnCode(filename, xmlString,
							  xmlLines, 'SEND_EVENT')
	
def getEventFileReceiveReturnCode(filename, xmlString, xmlLines):

	return getEventReturnCode(filename, xmlString,
							  xmlLines, 'RECEIVE_EVENT')
	
def getEventReturnCode(filename, xmlString, xmlLines, type):

	(doc, start) = parseXML(filename, xmlString, xmlLines)
	if doc != None:
		resultElement = getResultElements(None, start, type)
		if resultElement != None:
			return getReturnCode(resultElement)
			
	return None

# Returns true if the XML result contains ClientError element
def hasClientErrorElement(filename, xmlString, xmlLines):

	(doc, start) = parseXML(filename, xmlString, xmlLines)
	if doc != None:
		elements = start.getElementsByTagName('ClientErrorElement')
		if len[elements]:
			return 1

	return 0

# Return the exit code or None means no error at all
def getErrorExitCode(filename, xmlString, xmlLines):

	(doc, start) = parseXML(filename, xmlString, xmlLines)
	if doc != None:
		elements = start.getElementsByTagName('ClientError')
		if len(elements):
			return getReturnCode(elements[0])

	return None
	
# print `parseShowConfigXMLResult('/tmp/testXML', None)`
