import os
from LogLevel import *
from os.path import *
from SourceData import *
from Config import *

class Source:

	# Indicate the Source is a command-line type
	COMMAND = 0

	# Indicate the Source is a executable script file
	SCRIPT = 1

	# Indicate the Source contain file(s)
	FILES = 2

	# If the client and daemon are not in the same host,
	# it needs to indicate whether the SCRIPT or FILES
	# are located in client or server side
	serverSide = None
	
	type = None

	# This is only used as a script file
	args = None

	# This is used by SCRIPT or COMMAND
	# In COMMAND mode, cmdString includes all the arguments
	cmdString = None

	# Need to know this apply to local or remote
	local = 0

	# Use compression on fileList
	compressFiles = None
	
	# fileList - this is used by SCRIPT and FILES
	# A list of SourceData object
	fileList = None

	# This is only used in FILES mode. The list of files
	# can be derived from serveral dir
	dirList = None
	
	def __init__(self):
		self.type = type
		self.compressFiles = GlobalConfig.compressFiles

	def isCommand(self):
		if self.type == Source.COMMAND:
			return 1
		return 0

	def getCmdString(self):
		return self.cmdString

	def isScript(self):
		if self.type == Source.COMMAND:
			return 1
		return 0

	def isFile(self):
		if self.type == Source.FILES:
			return 1
		return 0

	def setLocal(self):
		self.local = 1

	def setRemote(self):
		self.local = 0
		
	def setCommand(self, cmdString):
		self.type = Source.COMMAND
		
		if cmdString:
			self.cmdString = cmdString
		else:
			GlobalConfig.log.write("Invalid command\n", LogLevel.ERROR)

	def setScript(self, scriptFile, *args):
		self.type = Source.SCRIPT

		if scriptFile:
			self.cmdString = scriptFile
		else:
			GlobalConfig.log.write("Invalid script file\n", LogLevel.ERROR)

		self.args = []
		for x in args:
			self.args.append(x)

	# The list can be either list of files or list of
	# directories
	# Note that this routine doesn't validate the files
	# or dirs immediately. This is only done when
	# validSource is called either from client or on
	# server side.
	def setFiles(self, list):
		self.type = Source.FILES

		self.fileList = list
		self.dirList = []

	def getFiles(self):
		for x in self.fileList:
			if x.isCompressed():
				x.uncompress()
		return self.fileList

	def checkAccess(self, file):
		if not os.access(file, os.R_OK):
			if isdir(file):
				fType = 'Dir: '
			else:
				fType = 'File: '
			GlobalConfig.log.write(fType + file + ' not included.' +
						   'No read permission\n',
						   LogLevel.ERROR)
			return 0

		return 1
	
	# Called by validateDir and validateSource.
	# Insert a valid SourceData object into self.fileList
	def validateFile(self, file, startdir):
	   	self.fileList.append(SourceData(file, startdir))

	# Recursive func on dir.
	def validateDir(self, dir, startdir):
		
		if self.checkAccess(dir):
			fileList = os.listdir(dir)
		else:
			errorMsg = 'No access permission for directory ' + dir + '\n'
			GlobalConfig.log.write(errorMsg, LogLevel.BAD_ERROR)
			return (0, errorMsg)
		
		msgList = []
		returnCode = 1
		
		for file in fileList:
   			file = join(dir,file)
			msg = ''
			messages = []
			if isfile(file):
				self.validateFile(file, startdir)
				rtCode, msg = 1, ''				
			elif isdir(file):
				rtCode, messages = self.validateDir(file, startdir)
				self.dirList.append(file)
			else:
				msg = (file + ' not included: not a regular' +
			           ' file nor directory\n')
				rtCode = 0
				GlobalConfig.log.write(msg, LogLevel.ERROR)

			if not rtCode:
				returnCode = 0
				if msg:
					msgList.append(msg)
				if messages:
					msgList.extend(messages)

		return returnCode, msgList
	
	# This routine is called by either client or server
	# based on the value of serverSide
	# Returns (0, msg) if error, otherwise returns (1, msg)
	def validateSource(self):

		msgList = []
		
		if self.type == Source.COMMAND:
			return 1
		
		elif self.type == Source.SCRIPT:
			self.fileList = []
			if isfile(self.cmdString):
				self.validateFile(self.cmdString)
			else:
				GlobalConfig.log.write("Cannot find script file or not" +
							   " a text file\n", LogLevel.ERROR)

		elif self.type == Source.FILES:
			tempFileList = self.fileList
			# print tempFileList
			self.fileList = []
			msgList = []
			returnCode = 1

			for item in tempFileList:
   				msg = ''
				messages = []
				if isfile(item):
					if os.access(item, os.R_OK):
						# print item
						startdir, filename = os.path.split(item)
					self.validateFile(item, startdir)
					rtCode, msg = 1, ''
				elif isdir(item):
					rtCode, messages = self.validateDir(item, item)
				else:
					rtCode = 0
					msg = (item + ' not included: not a regular' +
				           ' file nor directory\n')
					GlobalConfig.log.write(msg, LogLevel.ERROR)

				if not rtCode:
					returnCode = 0
				if msg:
					msgList.append(msg)
				if messages:
					msgList.extend(messages)

		# After all the validation. If there is/are SourceData objects,
		# then check whether the compress flag is set. If so, compress
		# them.
		if len(self.fileList):
			if self.compressFiles:
				for x in self.fileList:
					x.compress()
			return returnCode, msgList
		else:
			errorMsg = 'No file is included in the Source object\n'
			msgList.append(errorMsg)
			GlobalConfig.log.write(errorMsg, LogLevel.BAD_ERROR)
			return 0, msgList

	# Only pack data into blocks if we only got SourceData object
	# int the fileList
	def dataToBlocks(self):
		for x in self.fileList:
			if isinstance(x, SourceData):
				x.packBlocks()

	# Unpack the data into blocks				
	def blocksToData(self):
		for x in self.fileList:
			if isinstance(x, SourceData):
				x.unpackBlocks()

	# This is to speed up the delivery of the return msg
	def clearSourceData(self):
		for x in self.fileList:
			if isinstance(x, SourceData):
				x.clearData()
